@@ -1,7 +1,7 @@
 #
 #       images.R
 #
-#      $Revision: 1.170 $     $Date: 2021/07/17 02:44:48 $
+#      $Revision: 1.174 $     $Date: 2022/06/09 04:47:04 $
 #
 #      The class "im" of raster images
 #
@@ -788,7 +788,9 @@ safelookup <- function(Z, x, factor=2, warn=TRUE) {
   Zvals[isna] <- Nvals
   if(all(fixed))
     return(Zvals)
-  isna[isna] <- !fixed
+  notfixed <- !fixed
+  isna[isna] <- notfixed
+  Xbad <- Xbad[notfixed]
   #' Third pass - project to nearest pixel at any distance
   W <- as.mask(Z)
   eW <- exactPdt(W)
@@ -1081,8 +1083,13 @@ integral <- function(f, domain=NULL, ...) {
   UseMethod("integral")
 }
 
-integral.im <- function(f, domain=NULL, ...) {
+integral.im <- function(f, domain=NULL, weight=NULL, ...) {
   verifyclass(f, "im")
+  if(!is.null(weight)) {
+    if(is.function(weight))
+      weight <- as.im(weight, W=as.owin(f))
+    f <- f * weight
+  }
   typ <- f$type
   if(!any(typ == c("integer", "real", "complex", "logical")))
     stop(paste("Don't know how to integrate an image of type", sQuote(typ)))
